#!/usr/bin/env python


"""
Command line utility to process Alternating Grandient Force data.

     Copyright (C) 2005  Kurt Schwehr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
__version__ = '$Revision: 1.27 $'.split()[1] # See man ident
__date__ = '$Date: 2005/10/29 18:39:38 $'.split()[1]
__author__ = 'Kurt Schwehr'


import math,sys
from pmag_verbosity import BOMBASTIC,VERBOSE,TRACE,TERSE,ALWAYS
from pmag_kds_version import VERSION



def sqlCreate(tableName='agfm'):
    """
    Return an SQL CREATE string to build a table for agfm data

    I currently have no idea what man of these fields mean
    """

    # FIX: use the create function in pmag_sqlhelp
    s = 'CREATE TABLE ' +tableName + ' ('
    s+= ' key INTEGER PRIMARY KEY'
    s+= ', samplename VARCHAR(100)'  # There is no easy place to put sample name.  Part of the comment?
    s+= ', filename VARCHAR(255)' # Not a part of the header, but the file location and name
    s+= ', instrument VARCHAR(50)'

    # FIX: fill in all the fields

    #s+= ' ,' # 1 - position
    #s+= ' ,' # 2
    #s+= ' ,' # 3
    #s+= ' ,' # 4
    #s+= ' ,' # 5
    #s+= ' ,' # 6
    s+= ' step REAL,' # 7
    #s+= ' ,' # 8
    s+= ' range,' # 9
    #s+= ' ,' # 10
    #s+= ' ,' # 11
    #s+= ' ,'
    #s+= ' ,'
    #s+= ' ,'
    #s+= ' ,'
    #s+= ' ,'
    #s+= ' ,'
    #s+= ' ,'
    #s+= ' ,'
    #s+= ' ,'
    #s+= ' ,'
    #s+= ' ,'
    s+= ', comment VARHCHAR(80)' # FIX: what is the max length?  Is it 80 chars?
    #s+= ' ,'
    #s+= ' ,'
    #s+= ' ,'
    #s+= ' ,'
    s+= ', date VARCHAR(100)'  # FIX: make this a timestamp!
    #s+= ' ,'
    #s+= ' ,'
    #s+= ' ,'
    #s+= ' ,'
    #s+= ' ,'

    s+= ', s  REAL'
    s+= ', Ms REAL'
    s+= ', Mr REAL'
    s+= ', Bc REAL'
    s+= ', Bcr REAL'
    s+= ', Bcrn REAL'
    s+= ', sid VARCHAR(30)'  # Is this sample id?  It seams to be blank

    s+= ' );'
    return s



#import agfm
#from agfm import rows2cols
from Scientific.Functions.LeastSquares import leastSquaresFit

def linear(parameters,values):
    a,b = parameters
    x = values
    return (a*x + b)


def flattenHysteresis(sampleData,distanceCounts=None, distancePercent=None,
                          verbosity=0):
    """
    Detrend a hysteresis loop to remove the slope at high fields.

    FIX: write a better description of what this really means

    sampleData - agfm data (sample.data) in [ (x,y),(x,y) ... ]
    distanceCounts - how many samples you want to use on each tail
    distancePercent - [0..1] percent distance from the end that you want for correction

    Mr is the y intercept
    Ms is the saturation value
    Bc is the x axis.

    you get bcr by:
    subract ascending/descending loops
    b at which delta m is .5 of initial value is bcr

    """

    distance = None
    if distanceCounts == None and distancePercent == None:
        print 'ERROR: do not specify both distance methods!'
        print 'ERROR: FIX: is there a way in python to say what function I am in?'
        return None # FIX: throw exception
    if (distanceCounts != None) and (distancePercent != None):
        print 'ERROR: please pick a distance method'
        print 'ERROR: FIX: is there a way in python to say what function I am in?'
        return None # FIX: throw exception
    if distanceCounts != None: distance = distanceCounts
    else:
        assert(distancePercent>0. and distancePercent<1.)
        half = len(sample.data)/2
        distance = int(half * distancePercent)
        if verbosity>=VERBOSE: print 'percent = ', distance, half, options.distancePercent
        if distance < 4:
            print 'DANGER WILL ROBINSON!!!\n Can not fit a line to a point... incrementing distance'
            distance += 3
    if verbosity >= TRACE: print 'Using distance of '+str(distance)+' samples for correction factor'

    data = sample.data
    params=[]

    d = data[0:distance] + data[-distance:-1]
    if verbosity >= BOMBASTIC*2: print 'Fitting this data', d
    initial = (1,0)
    param,error = leastSquaresFit(linear,initial,d)
    params.append(param)

    #
    # CENTER is the left hand side of the graph
    #

    center = len(data)/2
    leftData = data[center-distance:center+distance]
    param,error = leastSquaresFit(linear,initial,leftData)
    params.append(param)
    dataLeft = rows2cols(leftData)

    a = 0;
    for i in range(len(params)): a += params[i][0]
    slope = a/len(params)
    if verbosity > TRACE: print 'average slope:  a = ',slope

    detrendedData = [ (pt[0], pt[1] - pt[0]*slope) for pt in data ]
    return detrendedData,slope

def getSlope(data):
    """
    Find the slope for a series of data points

    [(x1,y1),(x2,y2),(x3,y3),...(xn,yn)]

    Uses a leastSquaresFit from scientific python to y = mx + b
    """
    initial = (1,0) # FIX: maybe use the most distance points to pick the start
    param, error = leastSquaresFit(linear,initial,data)
    m = param[0]
    #print param, '   m = ',m
    return m

class agfm:
    "MicroMag 2900 running NewMag Windows 1995"
    def __init__(self, filename, doHystCrunch=True, doLocalHystCrunch=False, verbosity=0):
        '''
        Parse the header and the data.

        filename - read the data from here!
        doHystCrunch - do we crunch the data to produce Mr, Ms, etc?
        doLocalHystCrunch - use the code in this module to calculate the hysteresis values

        One big line of fields followined by field and value pairs

        e.g.

"Model 2900 ASCII Data File",7,-1,+2.500000E-01,+4.506305E+02,2,+0.000000E+00,+1.000000E+02,+1.000000E+04,+4.000000E+03,+0.000000E+00,0,82,164,+5.000000E-04,16,+4.833203E+01,+4.507500E+02,-8.161081E+01,+2.549891E-01,+4.703352E+01,3,"bp04-4gw-s1-063",+0.000000E+00,+0.000000E+00,+0.000000E+00,-1.000000E+03,"10/11/2005  11:02",0,0,0,0

+4.000500E+03,+2.925250E-04
+3.978000E+03,+2.916750E-04
+3.884500E+03,+2.867250E-04
+3.784500E+03,+2.811250E-04
...
+3.906500E+03,+2.879250E-04
+4.000500E+03,+2.933750E-04
"Model 2900 Data File ends"
        '''
        self.verbosity = verbosity
        self.filename = filename   #FIX: this is redundant wrt header... remove this one

        file = open(filename,'r')
        line = file.readline()
        fields = line.split(',')
        assert('"Model 2900 ASCII Data File"'==fields[0])
        self.header={}
        self.header['filename'] = filename
        self.header['instrument'] = (fields[0])[1:-1]
        i = 1
        #self.header[i] = fields[1];
        i+=1  # FIX: integer 7?
        #self.header[i] = fields[i];
        i+=1  # -1 ?
        #self.header[i] = fields[i];
        i+=1  # .25 ??
        #self.header[i] = fields[i];
        i+=1  # Q ?
        #self.header[i] = fields[i];
        i+=1  # 2?
        #self.header[i] = fields[i];

        i+=1  # 0?
        #print 'a',i, fields[i]
        self.header['step'] = float(fields[i]); # Field 7 - Applied field step size in Oe

        i+=1
        #print 'b',i, fields[i]
        #self.header[i] = fields[i];

        i+=1
        #print 'c',i, fields[i]
        self.header['range'] = float(fields[i]); # Field 9 - Applied field range in Oe

        i+=1
        #print i, fields[i]
        #self.header[i] = fields[i];

        i+=1
        #self.header[i] = fields[i];
        i+=1
        #self.header[i] = fields[i];
        i+=1
        #self.header[i] = fields[i];
        i+=1
        #self.header[i] = fields[i];
        i+=1
        #self.header[i] = fields[i];
        i+=1
        #self.header[i] = fields[i];
        i+=1
        #self.header[i] = fields[i];
        i+=1
        #self.header[i] = fields[i];
        i+=1
        #self.header[i] = fields[i];
        i+=1; 
        #self.header[i] = fields[i];
        i+=1
        #self.header[i] = fields[i];
        i+=1
        self.header['comment'] = fields[i][1:-1];  # Trim "" quotes
        i+=1
        #self.header[i] = fields[i];
        i+=1
        #self.header[i] = fields[i];
        i+=1
        #self.header[i] = fields[i];
        i+=1
        #self.header[i] = fields[i];
        i+=1
        # Strip off the quotes.  FIX: really handle the date/time
        self.header['date'] = (fields[i])[1:-1];
        i+=1
        #self.header[i] = fields[i];
        i+=1
        #self.header[i] = fields[i];
        i+=1
        #self.header[i] = fields[i];
        i+=1
        #self.header[i] = str(fields[i]).rstrip() # remove \r\n from last field

        self.data=[]
        for line in file.readlines():
            line = line.rstrip()
            if len(line)<3: continue
            if '"Model 2900 Data File ends"' == line: break # we is done
            fields = line.split(',')
            self.data.append((float(fields[0]),float(fields[1])))

        if doHystCrunch and doLocalHystCrunch:
            sys.exit("ERROR: Unable to cope with both hystcrunch and findHystParams") # FIX: throw exception
        if doHystCrunch:
            crunch = hystcrunch(filename)
            for key in crunch.keys():
                self.header[key] = crunch[key]
        elif doLocalHystCrunch:
            params = findHystParams(self.data)
            for key in params.keys():
                self.header[key] = param[key]
        return

    def plot(self,filename=None):
        """
        use matplotlib to make a simple graph of the data in the plot
        """
        import pylab
        cols = rows2cols(self.data)
        pylab.plot (cols[0],cols[1])
        pylab.title(self.header['comment'])
        if None==filename: pylab.show()
        else: pylab.savefig(filename)

    def getSqlInsertStr(self,table='hyst'):
        """
        Get the insert string
        """
        return self.getSqlInsert(table)
        
    def getSqlInsert(self,table='hyst'):
        """
        Return a SQL Insert object suitable for inserting this sample header info into a database

        Note that this leaves out the data block
        """
        from pmag_sqlhelp import sqlinsert
        ins = sqlinsert(table)
        for key in self.header.keys():
            if str != type(key):
                if self.verbosity>1:
                    print 'skipping key of none str type: ',key
                    continue
            ins.add(key,self.header[key])
        #import pdb
        #pdb.set_trace()

        return ins 

        

def hystcrunch(filename, adjustslope=False,verbosity=0):
    """
    Use pmag >= 1.9 fortran hystcrunch to calculate hysteresis parameters.
    Requires hystcrunch in the path

    sid,s,Ms,Mr*Ms,Bc,Bcr,Bcrn*1e-3

    Mr*Ms is really Mr?
    """
    import popen2
    cmd = "hystcrunch -pm"
    if not adjustslope: cmd+='a'
    if verbosity>1: print 'running command: '+cmd
    fin,fout=popen2.popen2(cmd)
    for line in open(filename,'r'):
        fout.write(line)
    fout.close()
    line = fin.readline()
    #sid,
    sid = ' '  # FIX: deal with when sid != white space
    s,Ms,Mr,Bc,Bcr,Bcrn = line.split()
    results = {
        'sid':sid,'s':float(s),'Ms':float(Ms),'Mr':float(Mr),'Bc':float(Bc),'Bcr':float(Bcr),'Bcrn':float(Bcrn)
    }
    return results

def rows2cols(list):
    """
    convert a big list of rows to list of columns.

    [ [1,2], [3,4], [5,6] ] to  [ [1,3,5], [2,4,6] ]
    """
    newlist = []
    for i in range(len(list[0])): newlist.append([])
    for row in list:
        for i in range(len(list[0])):
            newlist[i].append(row[i])
    return newlist

def lineFrom2Pts(pt1,pt2):
    """Return slope (m) and y intercept (b) for the line that fits two points"""
    if pt2[0] == pt1[0]:
        import decimal
        return decimal.Inf,0
    m = (pt2[1] - pt1[1])/ float(pt2[0]-pt1[0])
    b = pt1[1] - m * pt1[0]
    return m,b


import units # Unit coverstions
def findHystParams(data,verbosity=0):
    """
    Calculate hysteresis parameters from a flattened loop

    data in in Oe, while output is SI.  FIX: Is it really Oe going in?

    Pass in data = [(4000.5,5.939e-05), (3978.0,5.986e-05), ...]

    returns Mr, Ms, Bc, hopefully Bcr in a dict


    Looking at agfmcrunch, Ms is the y intercept of the unflattened data.
    Is that equiv to the flattened max?
    see agfmcrunch.f:linreg

    	call linreg(nreg,x,y,slope,Ms)
	subroutine linreg(n,x,y,s,b)

      Mr is the y intercept
      Ms is the saturation value
      Bc is the x axis.

      you get bcr by:
      subract ascending/descending loops
      b at which delta m is .5 of initial value is bcr

    """
    values = {}
    center = len(data)/2

    # Ms = saturation
    Ms_values = [data[0][1], data[-1][1], data[center-1][1],data[center][1] ]
    Ms_sum = 0;
    for item in Ms_values: Ms_sum+=math.fabs(item)
    Ms = Ms_sum / len(Ms_values)
    values['Ms'] = units.m_cgs2si(Ms) # emu to Am^2 # units.H_cgs2si(Ms)
    
    #f = open('foo','w')
    #for i in data:
    #    f.write(str(i[0])+' '+str(i[1])+'\n')

    ctr1stQtr = len(data)/4
    ctr3stQtr = (3*len(data))/4

    initial = (.01,0)
    param1,error = leastSquaresFit(linear,initial,data[ctr1stQtr-1:ctr1stQtr+2])
    param3,error = leastSquaresFit(linear,initial,data[ctr3stQtr-1:ctr3stQtr+2])
    Mr = (param1[1]-param3[1])/2  # Average the two Y intercepts.  FIX: is this ok?
    values['Mr'] = units.m_cgs2si(Mr) # emu to Am^2 # units.H_cgs2si(Ms)

    # Bc is the xaxis
    # Find the two points that cross x on the way down
    i = 0
    while data[i][1]>0: i += 1
    pt1 = data[i-1]
    pt2 = data[i]
    m,b = lineFrom2Pts(pt1,pt2)
    Bc1 = - b/m  # Crossing in Oe

    # Find the two points that cross x on the way up (2nd crossing)
    while data[i][1]<0: i += 1
    pt1 = data[i-1]
    pt2 = data[i]
    m,b = lineFrom2Pts(pt1,pt2)
    Bc2 = -b/m  # Crossing in Oe

    Bc = (math.fabs(Bc1) + math.fabs(Bc2)) / 2

    if verbosity>BOMBASTIC: print Bc1,Bc2,' -> ',Bc

    # GUESS NOT!!!  values['Bc'] = units.H_cgs2si(BcOe)  # Covert to A/m
    values['Bc'] = units.B_cgs2si(Bc)  # Covert to T from Gauss... FIX: is this really what is happening?


    return values

def printTodo():
    """
    Things that need to be done to this code:

    FIX: Better labeling of graphs
    FIX: Ability to close the hysteresis loops and center them vertically
    FIX: Plot out the 1st and 2nd derivatives
    FIX: Write out a nice report about the sample
    FIX: Take kappa bridge susceptability (low Chi, klf) and add to summary when --mass
    """
    print printTodo.__doc__


if __name__ == "__main__":
    from optparse import OptionParser

    myparser = OptionParser(usage="%prog [options] hyst1.raw hyst2.raw hyst3.raw ...",
                            version="%prog "+VERSION+' - '+__version__)
    myparser.add_option('--dump-header',dest='dumpHeader',default=False,action='store_true',
                        help='print out all the header fields for a file')
    myparser.add_option('--dump-values',dest='dumpData',default=False,action='store_true',
                        help='print out all the data values for a file')
    myparser.add_option('--show',dest='show',default=False,action='store_true',
                        help='Show a matplotlib graph of the loop')
    myparser.add_option('--show-flat',dest='showFlat',default=False,action='store_true',
                        help='Show a matplotlib graph of the loop after flattening')
    myparser.add_option('--plot-file',dest='plotFile',default=None,type='string',
                        help='Save plot to a file.  Filename extension determines the filetype.  '+
                        'e.g. .svg, .png, .ps [NOTE: .svg good for Illustrator and .ps broken(?)]')
    myparser.add_option('--plot-flat-file',dest='plotFlatFile',default=None,type='string',
                        help='Save plot to a file.  Filename extension determines the filetype.  '+
                        'e.g. .svg, .png')

    myparser.add_option('--sql-insert',dest='sqlInsert',default=False, action='store_true',
                        help='Generate a SQL INSERT statement that can be used for databases.')

    myparser.add_option('--sql-create',dest='sqlCreate',default=False, action='store_true',
                        help='Generate a SQL CREATE statement that can be used to make a new table')

    myparser.add_option('-t','--sql-table',dest='table',default='hyst',type='string',
                        help='Name of the database table. [default: %default]')


    myparser.add_option('--save-cgs',dest='saveCGS',default=False, action='store_true',
                        help='Save the cgs unflattened to a file')

    myparser.add_option('-s','--save-flattened',dest='saveFlattened',default=False, action='store_true',
                        help='Save the flattened loop to a file with .flat appended to the filename')

    
    myparser.add_option('--save-flattened-si',dest='saveFlattenedSI',default=False, action='store_true',
                        help='Save the flattened loop to a file with .flat.si appended to the filename.  '
                        + 'mT on the xaxis and Am^2 on the yaxis')

    myparser.add_option('--save-si',dest='saveSI',default=False, action='store_true',
                        help='Save the UNflattened loop to a file with .si appended to the filename.  '
                        + 'mT on the xaxis and Am^2 on the yaxis')


    myparser.add_option('-a','--agfm-format',dest='agfmFormat',default=False, action='store_true',
                        help='Change the flattened file format to be the AGFM Model 2900 output format,'
                        ' otherwise, it writes gnuplot compatable output')

    myparser.add_option('-D','--dist-counts',dest='distanceCounts',default=None, type='int',
                        help='How far from the edges to use for slope correction... using samples [default: %default]')
    myparser.add_option('-d','--dist-percent',dest='distancePercent',default=.05, type='float',
                        help='How far from the edges to use for slope correction... -D overrides this (0..1) [default: %default]')

    myparser.add_option('--xmin',dest='xmin',default=None,type='float',help='force plot range')
    myparser.add_option('--xmax',dest='xmax',default=None,type='float',help='force plot range')
    myparser.add_option('--ymin',dest='ymin',default=None,type='float',help='force plot range')
    myparser.add_option('--ymax',dest='ymax',default=None,type='float',help='force plot range')

    myparser.add_option('--ms-norm',dest='rescaleM',default=False, action='store_true',
                        help='Rescale M by dividing my Ms to get a +/- 1 plot')

    # FIX: allow reading the mass from the comment field.
    # FIX: allow volume to be specified in addition to mass
    myparser.add_option('--mass',dest='mass',default=None,type='float',
                        help='Mass in grams of the AGFM sample.  You did measure mass, right???')

    # FIX: calculate an average density from the cubes that is better....
    myparser.add_option('--density',dest='density',default=1.5,type='float',
                        help='Assumed density to calculate volume from mass.  In g/cm^3')

    myparser.add_option('--todo',dest='todo',default=False, action='store_true',
                        help='Print todo list')


    import pmag_verbosity
    pmag_verbosity.addVerbosityOptions(myparser)

    (options,args) = myparser.parse_args()

    if options.todo:
        printTodo()
        sys.exit()

    if None != options.xmin and None == options.xmax: sys.exit("ERROR: must specify both min and max for X")
    if None == options.xmin and None != options.xmax: sys.exit("ERROR: must specify both min and max for X")
    if None != options.ymin and None == options.ymax: sys.exit("ERROR: must specify both min and max for Y")
    if None == options.ymin and None != options.ymax: sys.exit("ERROR: must specify both min and max for Y")

    if options.sqlCreate:
        print sqlCreate(tableName=options.table)

    for filename in args:
        sample = agfm(filename)

        if options.dumpHeader:
            for key in sample.header.keys():
                keyStr = str(key)
                if len(keyStr) < 12:
                    keyStr = ' ' * (12 - len(keyStr)) + keyStr
                print keyStr+': ',sample.header[key]
        if options.dumpData:
            for i in sample.data:
                print i[0],i[1]
                
        if options.show:
            sample.plot()

        if options.sqlInsert:
            if options.verbosity>=TRACE: print 'sql insert string:'
            #import pdb
            #pdb.set_trace()

            print sample.getSqlInsertStr(table=options.table)

        dataFlattened,slope = None,None
        if options.distanceCounts != None:
            dataFlattened,slope = flattenHysteresis(sample.data,distanceCounts=options.distanceCounts,
                                            verbosity=options.verbosity)
        else:
            dataFlattened,slope = flattenHysteresis(sample.data,distancePercent=options.distancePercent,
                                            verbosity=options.verbosity)
        if options.verbosity>=TERSE: print 'slope = ',slope , '  emu/Oe'
        if options.verbosity>=BOMBASTIC+2: print 'flattened data',dataFlattened

        if options.mass != None:
            # Get the high field susceptibility (hiChi  k_hf, khf)
            flattenedData_cgs,slope_cgs = flattenHysteresis(sample.data,distancePercent=options.distancePercent,
                                               verbosity=options.verbosity)

            densityKg_m3 = options.density * 1000
            chiHighField = units.emuPerOe_2_volChiSI(slope_cgs,massGrams = options.mass,densityKg_m3 = densityKg_m3)
            print 'chiHighField(SI_vol_norm)',chiHighField

        #
        # FIX: merge the SI and normal flattened write into just one batch... this is currently lame
        # 

        if options.saveSI:
            dataSI = []
            print 'FIX: verify that CGS is correctly being converted to SI.  X-AXIS is wrong!!!'
            for line in sample.data:
                #Convert CGS to SI, but make it mT
                # xaxis: Oe to mT
                # FIX: 1000* on x axis???
                #dataFlatSI.append((units.H_cgs2si(line[0]),units.m_cgs2si(line[1]))) Thought it would be this
                # Oe -> mT  is *.1 ... this is what is in  hystcrunch fortran code
                # the m_cgs2si makes sense and gives me what I think
                # FIX: ????
                dataSI.append((0.1*line[0],units.m_cgs2si(line[1])))
            out = open (filename+'.si','w')
            if options.agfmFormat:
                print 'Not writing flattened SI in AGFM format... that would be a bad idea'
            # Always write out gnuplot format, since AGFM format would be bad... no way to know it is SI
            out.write ('# AGFM SI gnuplot stule\n')
            for key in sample.header.keys():
                out.write ('# '+str(key))
                if len(key) < 12:  out.write (' '*( 13 - len(key)))
                else: out.write(' ')
                out.write ('- '+str(sample.header[key])+'\n')
            # FIX: write out user name, date and what else?
            out.write ('# xaxis in mT\n')
            out.write ('# yaxis in Am^2\n')
            for line in dataSI:
                out.write(str(line[0])+' '+str(line[1])+'\n')
            out.close()

        if options.saveFlattenedSI:
            dataFlatSI = []
            print 'FIX: verify that CGS is correctly being converted to SI.  X-AXIS is wrong!!!'
            for line in dataFlattened:
                #Convert CGS to SI, but make it mT
                # xaxis: Oe to mT
                # FIX: 1000* on x axis???
                #dataFlatSI.append((units.H_cgs2si(line[0]),units.m_cgs2si(line[1])))
                # Oe -> mT  is *.1 ... this is what is in hystcrunch fortran code
                # the m_cgs2si makes sense and gives me what I think
                # FIX: ????
                dataFlatSI.append((0.1*line[0],units.m_cgs2si(line[1])))
            out = open (filename+'.flat.si','w')
            if options.agfmFormat:
                print 'Not writing flattened SI in AGFM format... that would be a bad idea'
            # Always write out gnuplot format, since AGFM format would be bad... no way to know it is SI
            out.write ('# AGFM SI gnuplot stule\n')
            for key in sample.header.keys():
                out.write ('# '+str(key))
                if len(key) < 12:  out.write (' '*( 13 - len(key)))
                else: out.write(' ')
                out.write ('- '+str(sample.header[key])+'\n')
            # FIX: write out user name, date and what else?
            out.write ('# xaxis in mT\n')
            out.write ('# yaxis in Am^2\n')
            for line in dataFlatSI:
                out.write(str(line[0])+' '+str(line[1])+'\n')
            out.close()

        if options.saveCGS:
            out = open (filename+'.cgs','w')
            for key in sample.header.keys():
                out.write ('# '+str(key))
                if len(key) < 12:  out.write (' '*( 13 - len(key)))
                else: out.write(' ')
                out.write ('- '+str(sample.header[key])+'\n')
                # FIX: write out user name, date and what else?
            for line in sample.data:
                out.write(str(line[0])+' '+str(line[1])+'\n')



        if options.saveFlattened:
            out = open (filename+'.flat','w')
            if options.agfmFormat:
                file = open(filename,'r')
                hdr = file.readline()
                file.close()
                if options.verbosity>VERBOSE: print 'hdr: ',hdr
                out.write(hdr+'\n') # Transplant

                for line in dataFlattened:
                    out.write(str(line[0])+','+str(line[1])+'\n')

                out.write('"Model 2900 Data File ends"\n')
            else:
                for key in sample.header.keys():
                    out.write ('# '+str(key))
                    if len(key) < 12:  out.write (' '*( 13 - len(key)))
                    else: out.write(' ')
                    out.write ('- '+str(sample.header[key])+'\n')
                    # FIX: write out user name, date and what else?
                for line in dataFlattened:
                    out.write(str(line[0])+' '+str(line[1])+'\n')
            out.close()

        hystparams = findHystParams(dataFlattened)
        print filename, hystparams

        # convert all data to si from cgs
        dataFlattened =  [ (x[0],units.m_cgs2si(x[1])) for x in dataFlattened]

        if options.rescaleM:
            # Normalize so that have M/Ms for [-1..1]
            # FIX: make sure the units are all the same!!!
            Ms = hystparams['Ms']
            #dataFlattened = [ (x[0],units.m_cgs2si(x[1])/Ms) for x in dataFlattened]
            dataFlattened = [ (x[0],x[1]/Ms) for x in dataFlattened]
        

        # FIX: should allow plotting to happen in cgs or si

        if options.showFlat or (options.plotFlatFile != None):
            if options.verbosity>=TRACE: print 'flat plotting...'
            d2cols = rows2cols(dataFlattened)
            import pylab
            pylab.plot (d2cols[0],d2cols[1], hold=False)
            axes = None
            if options.xmin or options.ymin: axis = pylab.axes()
            if options.xmin:
                print 'setting x axis to ', (options.xmin,options.xmax)
                axis.set_xlim((options.xmin,options.xmax))
            #print 'ylim = ',(options.ymin,options.ymax),type(options.ymin),type(options.ymax)
            if options.ymin: axis.set_ylim((options.ymin,options.ymax))
            pylab.xlabel('Applied field - (Gauss??)  B')
            if options.rescaleM: pylab.ylabel('M/Ms')
            else: pylab.ylabel('M (Am^2)')
            pylab.figtext (.55,.29  ,'slope = '+("%0.4e" % slope)+'  units',fontname='Courier')
            pylab.figtext (.55,.265 ,'Ms = '+("%0.4e" % hystparams['Ms'])+'  Am^2',fontname='Courier')
            pylab.figtext (.55,.24  ,'Mr = '+("%0.4e" % hystparams['Mr'])+'  Am^2',fontname='Courier')
            pylab.figtext (.55,.215 ,'Bc = '+("%0.4e" % hystparams['Bc'])+'  T',fontname='Courier')
            pylab.grid(True)
            if sample.header.has_key('samplename'): pylab.title(sample.header['samplename']+' flattened')
            else: pylab.title(filename)
            if options.showFlat: pylab.show()
            if options.plotFlatFile: pylab.savefig(options.plotFlatFile)

