#!/usr/bin/env python

"""
Create pdf with a composite of the images passed in

Pass in a bunch of images (png, gif, jpg, and ?) and have them all put
together a poster with all the images aranged on it.  This is not
meant to be beautiful, so it is up to you to edit the results in
something like Illustrator


     Copyright (C) 2005  Kurt Schwehr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

$Id: pdfposter.py,v 1.4 2005/10/29 15:31:06 schwehr Exp $
"""


# FIX: check out the size of the image and scale appropriately
# FIX: add text to each image
# FIX: how to do grouping?
# FIX: make a choice of paper sizes or explicite point sizes

# These values end up in the pydoc web page
__version__ = '$Revision: 1.4 $'.split()[1] # See man ident
__date__ = '$Date: 2005/10/29 15:31:06 $'.split()[1]
__author__ = 'Kurt Schwehr'
__credits__ = 'JWZ for doing something much like this in perl'

import math,sys
from pmag_verbosity import BOMBASTIC,VERBOSE,TRACE,TERSE,ALWAYS
from pmag_kds_version import VERSION
import datetime
from pdflib_py import *
#import Image		# PIL - python imaging library
#import ImageDraw #, ImageFont

points32Inches = 2304
points36Inches = 2592
points60Inches = 4320
points72Inches = 5184

pointsWide = points36Inches
pointsHigh = points60Inches

if __name__ == "__main__":
    from optparse import OptionParser

    myparser = OptionParser(usage="%prog [options] image1.png image2.gif image3.jpg ...",
                            version="%prog "+VERSION+' - '+__version__)
    myparser.add_option('-t','--title',dest='title',default=None,type='string',
                        help='Title to put at the top [default: %default]')
    myparser.add_option('-d','--date',dest='date',default=False,action='store_true',
                        help='Put the date in the title')
    myparser.add_option('-o','--out',dest='outFile',default='poster.pdf',type='string',
                        help='What to call the resulting pdf [default: %default]')
    myparser.add_option('-s','--scale',dest='scaleFactor',default=0.2,type='float',
                        help='Size of the images [default: %default]')
    myparser.add_option('-S','--spacing',dest='spacingFactor',default=210,type='int',
                        help='x offset for each column [default: %default]')
    import pmag_verbosity
    pmag_verbosity.addVerbosityOptions(myparser)

    (options,args) = myparser.parse_args()
    if (options.scaleFactor>10) or (options.scaleFactor<0.):
        sys.exit("ERROR: your scale factor seems unlikely to work: "+str(options.scaleFactor) )


    pdf = PDF_new()
    if -1 == PDF_open_file(pdf,options.outFile):
        system.exit('Unable to open new pdf file '+options.outFile)
    PDF_set_info(pdf, 'Creator', 'agfm-poster.py')
    PDF_set_info(pdf, 'Author', 'Kurt Schwehr')
    PDF_set_info(pdf, 'Title', 'AGFM of BPSIO-2004 cubes')


    PDF_begin_page(pdf,pointsWide,pointsHigh);

    font = PDF_load_font(pdf,"Helvetica-Bold","iso8859-1","")
    PDF_setfont(pdf,font, 42)
    PDF_set_text_pos(pdf,750,pointsHigh - 50)
    if options.title != None:
        title = options.title
        if options.date: title += ' ' + str(datetime.date.today())
        PDF_show(pdf,title);

    # FIX: make these based on emacs image... needs major cleanup
    sampleHeight = 210
    sampleWidth = options.spacingFactor
    pointsTop = pointsHigh - 150
    y = pointsTop
    y-=100
    columnTop = y
    x = 100 #sampleHeight # FIX: do something smarter??

    for file in args:

        print 'file = ',file
        if y < 0:
            print "Ran off the bot of the page!  Wrapping to the next column"
            y = columnTop
            x += sampleWidth
        print ' now at ',x,y
        pdfImage = PDF_load_image(pdf,"auto", file, "")

        if -1 != pdfImage:
            print "Putting image at ",x,y
            PDF_fit_image(pdf,pdfImage,x,y,"scale "+str(options.scaleFactor))
            PDF_close_image(pdf,pdfImage)
            y -= sampleHeight
        else: sys.exit("ERROR: unable to open " + file)

    PDF_end_page(pdf)
    PDF_close(pdf)
    PDF_delete(pdf)
